@@ -1,7 +1,7 @@
 package Object::props ;
-$VERSION = 1.2 ;
+$VERSION = 1.3 ;
 
-use base 'Base::OOTools' ;
+use base 'Class::props' ;
 
 1 ;
 
@@ -11,13 +11,13 @@ __END__
 
 Object::props - Pragma to implement lvalue accessors with options
 
-=head1 VERSION 1.2
+=head1 VERSION 1.3
 
-Included in OOTools 1.2 distribution. The distribution includes:
+Included in OOTools 1.3 distribution. The distribution includes:
 
 =over
 
-=item * Class::new
+=item * Class::constr
 
 Pragma to implement constructor methods
 
@@ -25,10 +25,18 @@ Pragma to implement constructor methods
 
 Pragma to implement lvalue accessors with options
 
+=item * Class::group
+
+Pragma to implement group of properties accessors with options
+
 =item * Object::props
 
 Pragma to implement lvalue accessors with options
 
+=item * Object::group
+
+Pragma to implement group of properties accessors with options
+
 =back
 
 =head1 SYNOPSIS
@@ -38,36 +46,37 @@ Pragma to implement lvalue accessors with options
     package MyClass ;
     
     # implement constructor without options
-    use Class::new ;
+    use Class::constr ;
     
     # just accessors without options (list of strings)
-    use Object::props @prop_names ;
+    use Object::props @prop_names ;                      # @prop_names (1)
     
     # a property with validation and default (list of hash refs)
     use Object::props { name       => 'digits',
-                        validation => sub{ /^\d+\z/ }       # just digits
+                        validation => sub{ /^\d+\z/ }    # just digits
                         default    => 10
                       } ;
     
     # a group of properties with common full options
-    use Object::props { name       => \@other_prop_names,   # array ref
+    use Object::props { name       => \@prop_names2,     # @prop_names2 (1)
                         rt_default => sub{$_[0]->other_default} ,
                         validation => sub{ /\w+/ }
                         protected  => 1
                       } ;
                       
     # all the above in just one step (list of strings and hash refs)
-    use Object::props @prop_names ,
+    use Object::props @prop_names ,                      # @prop_names (1)
                       { name       => 'digits',
                         validation => sub{ /^\d+\z/ }
                         default    => 10
                       } ,
-                      { name       => \@other_prop_names,
+                      { name       => \@prop_names2,     # @prop_names2 (1)
                         rt_default => sub{$_[0]->other_default} ,
                         validation => sub{ /\w+/ }
                         protected  => 1
                       } ;
-    
+                      
+    # (1) must be set in a BEGIN block to have effect at compile time
 
 =head2 Usage
 
@@ -92,7 +101,7 @@ This pragma easily implements lvalue accessor methods for the properties of your
 
 You can completely avoid to write the accessor by just declaring the names and eventually the default value, validation code and other option of your properties.
 
-The accessor method creates a key in the hash object that implements it (e.g. $object->{property_name}) and ties it to the options you set, so even if you access the key without using the accessor, the options will have effect.
+The accessor method creates a key in the hash object that implements it (e.g. $object->{property}) and ties it to the options you set, so even if you access the key without using the accessor, the options will have effect.
 
 =head2 Class properties vs Object properties
 
@@ -101,7 +110,7 @@ The main difference between C<Object::props> and C<Class::props> is that the fir
 A Class property is accessible either through the class or through all the objects of that class, while an object property is accessible only through the object that set it.
 
    package MyClass;
-   use Class::new ;
+   use Class::constr ;
    use Object::props 'obj_prop' ;
    use Class::props qw( class_prop1
                         class_prop2 ) ;
@@ -127,7 +136,7 @@ A Class property is accessible either through the class or through all the objec
    print $MyClass::class_prop2 ; # would print 22
    
    $object2->class_prop1 = 100 ; # object method
-   MyClass->class_prop2  = 200 ; # static method
+   MyClass->class_prop2  = 200 ; # static method works as well
    
    print $object1->class_prop1 ; # would print 100
    print $object2->class_prop1 ; # would print 100
@@ -170,8 +179,8 @@ Given 'my_prop' as the property name:
     $object->my_prop = 10 ;  # assign 10 to $object->{my_prop}
     $object->my_prop( 10 );  # assign 10 to $object->{my_prop}
     
-    # same thing if MyClass::new is implemented
-    # by the Class::new pragma
+    # same thing if MyClass::constr is implemented
+    # by the Class::constr pragma
     
     $object = MyClass->new( my_prop => 10 );
 
@@ -228,20 +237,11 @@ I would like to have just a line of feedback from everybody who tries or actuall
 
 More information at http://perl.4pro.net/?Object::props.
 
-=head1 AUTHOR
+=head1 AUTHOR and COPYRIGHT
 
-Domizio Demichelis, <dd@4pro.net>.
-
-=head1 COPYRIGHT
-
-Copyright (c)2002 Domizio Demichelis. All Rights Reserved. This is free software; it may be used freely and redistributed for free providing this copyright header remains part of the software. You may not charge for the redistribution of this software. Selling this code without Domizio Demichelis' written permission is expressly forbidden.
-
-This software may not be modified without first notifying the author (this is to enable me to track modifications). In all cases the copyright header should remain fully intact in all modifications.
-
-This code is provided on an "As Is'' basis, without warranty, expressed or implied. The author disclaims all warranties with regard to this software, including all implied warranties of merchantability and fitness, in no event shall the author, be liable for any special, indirect or consequential damages or any damages whatsoever including but not limited to loss of use, data or profits. By using this software you agree to indemnify the author from any liability that might arise from it is use. Should this code prove defective, you assume the cost of any and all necessary repairs, servicing, correction and any other costs arising directly or indrectly from it is use.
-
-The copyright notice must remain fully intact at all times. Use of this software or its output, constitutes acceptance of these terms.
+ 2003 by Domizio Demichelis <dd@4pro.net>.
 
+All Rights Reserved. This module is free software. It may be used, redistributed and/or modified under the same terms as perl itself.
 
 =head1 CREDITS
 
